"use strict";
// ================================ Imports ================================ //
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseOptions = exports.parseArguments = void 0;
const regexes_js_1 = __importDefault(require("./regexes.js"));
const utilities_js_1 = require("./utilities.js");
// =========================== Parsing Functions =========================== //
function parseArguments(spec, providedArgs) {
    const args = {};
    const keys = spec.match(regexes_js_1.default.argumentParse);
    if (!keys)
        return args;
    const required = (() => {
        const s = keys[1].trim().replace(/[<>]/g, "");
        const a = s.length > 0 ? s.split(" ") : [];
        return a;
    })();
    const last = keys[2]?.replace(/[[\]<>.]/g, "");
    required.forEach((key, index) => {
        if (!providedArgs[index])
            throw new Error(`Missing argument: ${key}`);
        (0, utilities_js_1.define)(args, key, providedArgs[index]);
    });
    if (last) {
        // required.length is passed in down here
        // because it's the last index of the
        // array + 1
        if (providedArgs[required.length]) {
            (0, utilities_js_1.define)(args, last, keys[2]?.includes("...") // Checks if last is variadic
                ? providedArgs.slice(required.length).join(" ").trimEnd()
                : providedArgs[required.length]);
        }
        else if (!/[[\]]/g.test(keys[2])) {
            // Checks if last is not optional
            throw new Error(`Missing argument: ${last}`);
        }
    }
    return args;
}
exports.parseArguments = parseArguments;
function parseOptions(exec, commandOptions) {
    const options = {};
    const stringified = exec.join(" ");
    const regexMatch = (0, utilities_js_1.matchAll)(regexes_js_1.default.optionParse, stringified);
    for (const match of regexMatch) {
        const optionKey = match[1];
        const optionMeta = commandOptions.find((e) => {
            return e.name === optionKey || e.alias === optionKey;
        });
        if (!optionMeta)
            continue;
        const optionKeyCamel = (0, utilities_js_1.camelCase)(optionMeta.name.substring(2)
            .replaceAll(".", " ")
            .replaceAll("-", " ")
            .trim()
            .split(" "));
        const defaultValue = optionMeta.flag ? false : optionMeta.default;
        const optionValue = optionMeta.flag || (match[2] || "").replace(/(^")|("$)/g, "");
        (0, utilities_js_1.define)(options, optionKeyCamel, optionValue || defaultValue);
        (0, utilities_js_1.define)(options, optionMeta.name, optionValue || defaultValue);
    }
    return options;
}
exports.parseOptions = parseOptions;
